@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2016 2ndQuadrant Italia Srl
+# Copyright (C) 2011-2017 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -385,11 +385,15 @@ class BackupManager(RemoteStatusMixin):
                          '\n'.join(msg_lines[1:]))
 
         else:
-            output.info("Backup end at xlog location: %s (%s, %08X)",
+            output.info("Backup end at LSN: %s (%s, %08X)",
                         backup_info.end_xlog,
                         backup_info.end_wal,
                         backup_info.end_offset)
-            output.info("Backup completed")
+            output.info("Backup completed (start time: %s, elapsed time: %s)",
+                        self.executor.copy_start_time,
+                        human_readable_timedelta(
+                            self.executor.copy_end_time
+                            - self.executor.copy_start_time))
             # Create a restore point after a backup
             target_name = 'barman_%s' % backup_info.backup_id
             self.server.postgres.create_restore_point(target_name)
@@ -423,7 +427,7 @@ class BackupManager(RemoteStatusMixin):
 
     def recover(self, backup_info, dest, tablespaces=None, target_tli=None,
                 target_time=None, target_xid=None, target_name=None,
-                exclusive=False, remote_command=None):
+                target_immediate=False, exclusive=False, remote_command=None):
         """
         Performs a recovery of a backup
 
@@ -436,6 +440,8 @@ class BackupManager(RemoteStatusMixin):
         :param str|None target_xid: the target xid
         :param str|None target_name: the target name created previously with
             pg_create_restore_point() function call
+        :param bool|None target_immediate: end recovery as soon as consistency
+            is reached
         :param bool exclusive: whether the recovery is exclusive or not
         :param str|None remote_command: default None. The remote command
             to recover the base backup, in case of remote backup.
@@ -449,7 +455,8 @@ class BackupManager(RemoteStatusMixin):
                                          dest, tablespaces,
                                          target_tli, target_time,
                                          target_xid, target_name,
-                                         exclusive, remote_command)
+                                         target_immediate, exclusive,
+                                         remote_command)
 
         # Output recovery results
         output.result('recovery', recovery_info['results'])
@@ -463,9 +470,8 @@ class BackupManager(RemoteStatusMixin):
 
         :param bool verbose: report even if no actions
         """
-        with self.server.xlogdb('a') as fxlogdb:
-            for archiver in self.server.archivers:
-                archiver.archive(fxlogdb, verbose)
+        for archiver in self.server.archivers:
+            archiver.archive(verbose)
 
     def cron_retention_policy(self):
         """
@@ -545,10 +551,10 @@ class BackupManager(RemoteStatusMixin):
         :param CheckStrategy check_strategy: the strategy for the management
              of the results of the various checks
         """
+        check_strategy.init_check('compression settings')
         # Check compression_setting parameter
         if self.config.compression and not self.compression_manager.check():
-            check_strategy.result(self.config.name,
-                                  'compression settings', False)
+            check_strategy.result(self.config.name, False)
         else:
             status = True
             try:
@@ -557,19 +563,18 @@ class BackupManager(RemoteStatusMixin):
                 check_strategy.result(self.config.name,
                                       '%s setting' % field, False)
                 status = False
-            check_strategy.result(self.config.name,
-                                  'compression settings', status)
+            check_strategy.result(self.config.name, status)
 
         # Failed backups check
+        check_strategy.init_check('failed backups')
         failed_backups = self.get_available_backups((BackupInfo.FAILED,))
         status = len(failed_backups) == 0
         check_strategy.result(
             self.config.name,
-            'failed backups',
             status,
-            'there are %s failed backups' % (len(failed_backups,))
+            hint='there are %s failed backups' % (len(failed_backups,))
         )
-
+        check_strategy.init_check('minimum redundancy requirements')
         # Minimum redundancy checks
         no_backups = len(self.get_available_backups())
         # Check minimum_redundancy_requirements parameter
@@ -578,9 +583,8 @@ class BackupManager(RemoteStatusMixin):
         else:
             status = True
         check_strategy.result(
-            self.config.name,
-            'minimum redundancy requirements', status,
-            'have %s backups, expected at least %s' % (
+            self.config.name, status,
+            hint='have %s backups, expected at least %s' % (
                 no_backups, self.config.minimum_redundancy))
 
         # TODO: Add a check for the existence of ssh and of rsync
@@ -675,6 +679,12 @@ class BackupManager(RemoteStatusMixin):
                                     wal_count += 1
                                 elif xlog.is_backup_file(fullname):
                                     label_count += 1
+                                elif fullname.endswith('.tmp'):
+                                    _logger.warning(
+                                        'temporary file found '
+                                        'rebuilding the wal database: %s',
+                                        fullname)
+                                    continue
                                 else:
                                     _logger.warning(
                                         'unexpected file '
@@ -775,7 +785,7 @@ class BackupManager(RemoteStatusMixin):
                     wal_info = WalFileInfo.from_xlogdb_line(line)
                     if not xlog.is_any_xlog_file(wal_info.name):
                         output.error(
-                            "invalid xlog segment name %r\n"
+                            "invalid WAL segment name %r\n"
                             "HINT: Please run \"barman rebuild-xlogdb %s\" "
                             "to solve this issue",
                             wal_info.name, self.config.name)
