@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2016 2ndQuadrant Italia Srl
+# Copyright (C) 2011-2017 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -24,12 +24,14 @@ import os
 import sys
 from argparse import SUPPRESS, ArgumentTypeError
 from contextlib import closing
+from functools import wraps
 
 from argh import ArghParser, arg, expects_obj, named
 
 import barman.config
 import barman.diagnose
 from barman import output
+from barman.config import RecoveryOptions
 from barman.exceptions import BadXlogSegmentName
 from barman.infofile import BackupInfo
 from barman.server import Server
@@ -183,6 +185,9 @@ def backup_completer(prefix, parsed_args, **kwargs):
      type=check_non_negative)
 @arg('--no-retry', help='Disable base backup copy retry logic.',
      dest='retry_times', action='store_const', const=0)
+@arg('--jobs', '-j',
+     help='Run the copy in parallel using NJOBS processes.',
+     type=check_positive, metavar='NJOBS')
 @expects_obj
 def backup(args):
     """
@@ -204,6 +209,8 @@ def backup(args):
             server.config.basebackup_retry_times = args.retry_times
         if hasattr(args, 'immediate_checkpoint'):
             server.config.immediate_checkpoint = args.immediate_checkpoint
+        if args.jobs is not None:
+            server.config.parallel_jobs = args.jobs
         with closing(server):
             server.backup()
     output.close_and_exit()
@@ -320,6 +327,10 @@ def rebuild_xlogdb(args):
 @arg('--target-name',
      help='target name created previously with '
           'pg_create_restore_point() function call')
+@arg('--target-immediate',
+     help='end recovery as soon as a consistent state is reached',
+     action='store_true',
+     default=False)
 @arg('--exclusive',
      help='set target xid to be non inclusive', action="store_true")
 @arg('--tablespace',
@@ -345,6 +356,29 @@ def rebuild_xlogdb(args):
      type=check_non_negative)
 @arg('--no-retry', help='Disable base backup copy retry logic.',
      dest='retry_times', action='store_const', const=0)
+@arg('--jobs', '-j',
+     help='Run the copy in parallel using NJOBS processes.',
+     type=check_positive, metavar='NJOBS')
+@arg('--get-wal',
+     help='Enable the get-wal option during the recovery.',
+     dest='get_wal',
+     action='store_true',
+     default=SUPPRESS)
+@arg('--no-get-wal',
+     help='Disable the get-wal option during recovery.',
+     dest='get_wal',
+     action='store_false',
+     default=SUPPRESS)
+@arg('--network-compression',
+     help='Enable network compression during remote recovery.',
+     dest='network_compression',
+     action='store_true',
+     default=SUPPRESS)
+@arg('--no-network-compression',
+     help='Disable network compression during remote recovery.',
+     dest='network_compression',
+     action='store_false',
+     default=SUPPRESS)
 @expects_obj
 def recover(args):
     """
@@ -399,6 +433,42 @@ def recover(args):
         server.config.basebackup_retry_sleep = args.retry_sleep
     if args.retry_times is not None:
         server.config.basebackup_retry_times = args.retry_times
+    if hasattr(args, 'get_wal'):
+        if args.get_wal:
+            server.config.recovery_options.add(RecoveryOptions.GET_WAL)
+        else:
+            server.config.recovery_options.remove(RecoveryOptions.GET_WAL)
+    if args.jobs is not None:
+        server.config.parallel_jobs = args.jobs
+
+    # PostgreSQL supports multiple parameters to specify when the recovery
+    # process will end, and in that case the last entry in recovery.conf
+    # will be used. See [1]
+    #
+    # Since the meaning of the target options is not dependent on the order
+    # of parameters, we decided to make the target options mutually exclusive.
+    #
+    # [1]: https://www.postgresql.org/docs/current/static/
+    #   recovery-target-settings.html
+
+    target_options = ['target_tli', 'target_time', 'target_xid',
+                      'target_name', 'target_immediate']
+    specified_target_options = len(
+        [option for option in target_options if getattr(args, option)])
+    if specified_target_options > 1:
+        output.error(
+            "You cannot specify multiple targets for the recovery operation")
+        output.close_and_exit()
+
+    if hasattr(args, 'network_compression'):
+        if args.network_compression and args.remote_ssh_command is None:
+            output.error(
+                "Network compression can only be used with "
+                "remote recovery.\n"
+                "HINT: If you want to do a remote recovery "
+                "you have to use the --remote-ssh-command option")
+            output.close_and_exit()
+        server.config.network_compression = args.network_compression
     with closing(server):
         server.recover(backup_id,
                        args.destination_directory,
@@ -407,6 +477,7 @@ def recover(args):
                        target_time=args.target_time,
                        target_xid=args.target_xid,
                        target_name=args.target_name,
+                       target_immediate=args.target_immediate,
                        exclusive=args.exclusive,
                        remote_command=args.remote_ssh_command)
 
@@ -445,30 +516,30 @@ def show_server(args):
     output.close_and_exit()
 
 
-@named('switch-xlog')
+@named('switch-wal')
 @arg('server_name', nargs='+',
      completer=server_completer_all,
-     help="specifies the server name target of the switch-xlog command")
+     help="specifies the server name target of the switch-wal command")
 @arg('--force',
-     help='forces the switch of a xlog by executing a checkpoint before',
+     help='forces the switch of a WAL by executing a checkpoint before',
      dest='force',
      action='store_true',
      default=False)
 @arg('--archive',
-     help='wait for one xlog file to be archived',
+     help='wait for one WAL file to be archived',
      dest='archive',
      action='store_true',
      default=False)
 @arg('--archive-timeout',
-     help='the time, in seconds, the archiver will wait for a new xlog file '
+     help='the time, in seconds, the archiver will wait for a new WAL file '
           'to be archived before timing out',
      metavar='TIMEOUT',
      default='30',
      type=check_non_negative)
 @expects_obj
-def switch_xlog(args):
+def switch_wal(args):
     """
-    Execute the switch-xlog command on the target server
+    Execute the switch-wal command on the target server
     """
     servers = get_server_list(args, skip_inactive=True)
     for name in sorted(servers):
@@ -477,10 +548,20 @@ def switch_xlog(args):
         if not manage_server_command(server, name):
             continue
         with closing(server):
-            server.switch_xlog(args.force, args.archive, args.archive_timeout)
+            server.switch_wal(args.force, args.archive, args.archive_timeout)
     output.close_and_exit()
 
 
+@named('switch-xlog')
+# Set switch-xlog as alias of switch-wal.
+# We cannot use the @argh.aliases decorator, because it needs Python >= 3.2,
+# so we create a wraqpper function and use @wraps to copy all the function
+# attributes needed by argh
+@wraps(switch_wal)
+def switch_xlog(args):
+    return switch_wal(args)
+
+
 @arg('server_name', nargs='+',
      completer=server_completer_all,
      help="specifies the server names to check "
@@ -1033,6 +1114,7 @@ def main():
             show_server,
             replication_status,
             status,
+            switch_wal,
             switch_xlog,
         ]
     )
