@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2016 2ndQuadrant Italia Srl
+# Copyright (C) 2011-2017 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -113,7 +113,7 @@ class Command(object):
         The subprocess output and error stream will be processed through
         the output and error handler, respectively defined through the
         `out_handler` and `err_handler` arguments. If not provided every line
-        will be sent to teh log respectively at INFO and WARNING level.
+        will be sent to the log respectively at INFO and WARNING level.
 
         The `out_handler` and the `err_handler` functions will be invoked with
         one single argument, which is a string containing the line that is
@@ -219,18 +219,6 @@ class Command(object):
         """restore default signal handler (http://bugs.python.org/issue1652)"""
         signal.signal(signal.SIGPIPE, signal.SIG_DFL)  # pragma: no cover
 
-    @staticmethod
-    def _cmd_quote(cmd, args):
-        """
-        Quote all cmd's arguments.
-
-        This is needed to avoid command string breaking.
-        WARNING: this function does not protect against injection.
-        """
-        if args is not None and len(args) > 0:
-            cmd = "%s '%s'" % (cmd, "' '".join(args))
-        return cmd
-
     def __call__(self, *args, **kwargs):
         """
         Run the command and return the exit code.
@@ -377,7 +365,7 @@ class Command(object):
         The subprocess output and error stream will be processed through
         the output and error handler, respectively defined through the
         `out_handler` and `err_handler` arguments. If not provided every line
-        will be sent to teh log respectively at INFO and WARNING level.
+        will be sent to the log respectively at INFO and WARNING level.
 
         If the `close_fds` argument is True, all file descriptors
         except 0, 1 and 2 will be closed before the child process is executed.
@@ -458,7 +446,7 @@ class Command(object):
         args = self.args + list(args)
         # If shell is True, properly quote the command
         if self.shell:
-            cmd = self._cmd_quote(self.cmd, args)
+            cmd = full_command_quote(self.cmd, args)
         else:
             cmd = [self.cmd] + args
 
@@ -605,7 +593,7 @@ class Rsync(Command):
         """
         options = []
         if ssh:
-            options += ['-e', self._cmd_quote(ssh, ssh_options)]
+            options += ['-e', full_command_quote(ssh, ssh_options)]
         if network_compression:
             options += ['-z']
         # Include patterns must be before the exclude ones, because the exclude
@@ -691,7 +679,16 @@ class PostgreSQLClient(Command):
     Superclass of all the PostgreSQL client commands.
     """
 
-    COMMAND = None
+    COMMAND_ALTERNATIVES = None
+    """
+    Sometimes the name of a command has been changed during the PostgreSQL
+    evolution. I.e. that happened with pg_receivexlog, that has been renamed
+    to pg_receivewal. In that case, we should try using pg_receivewal (the
+    newer auternative) and, if that command doesn't exist, we should try
+    using `pg_receivewal`.
+
+    This is a list of command names to be used to find the installed command.
+    """
 
     def __init__(self,
                  connection,
@@ -730,6 +727,74 @@ class PostgreSQLClient(Command):
         self.enable_signal_forwarding(signal.SIGINT)
         self.enable_signal_forwarding(signal.SIGTERM)
 
+    @classmethod
+    def find_command(cls, path=None):
+        """
+        Find the active command, given all the alternatives as set in the
+        property named `COMMAND_ALTERNATIVES` in this class.
+
+        :param str path: The path to use while searching for the command
+        :rtype: Command
+        """
+
+        # TODO: Unit tests of this one
+
+        # To search for an available command, testing if the command
+        # exists in PATH is not sufficient. Debian will install wrappers for
+        # all commands, even if the real command doesn't work.
+        #
+        # I.e. we may have a wrapper for `pg_receivewal` even it PostgreSQL
+        # 10 isn't installed.
+        #
+        # This is an example of what can happen in this case:
+        #
+        # ```
+        # $ pg_receivewal --version; echo $?
+        # Error: pg_wrapper: pg_receivewal was not found in
+        #   /usr/lib/postgresql/9.6/bin
+        # 1
+        # $ pg_receivexlog --version; echo $?
+        # pg_receivexlog (PostgreSQL) 9.6.3
+        # 0
+        # ```
+        #
+        # That means we should not only ensure the existence of the command,
+        # but we also need to invoke the command to see if it is a shim
+        # or not.
+
+        # Get the system path if needed
+        if path is None:
+            path = os.getenv('PATH')
+        # If the path is None at this point we have nothing to search
+        if path is None:
+            path = ''
+
+        # Search the requested executable in every directory present
+        # in path and return a Command object first occurrence that exists,
+        # is executable and runs without errors.
+        for path_entry in path.split(os.path.pathsep):
+            for cmd in cls.COMMAND_ALTERNATIVES:
+                full_path = barman.utils.which(cmd, path_entry)
+
+                # It doesn't exist try another
+                if not full_path:
+                    continue
+
+                # It exists, let's try invoking it with `--version` to check if
+                # it's real or not.
+                try:
+                    command = Command(full_path, path=path, check=True)
+                    command("--version")
+                    return command
+                except CommandFailedException:
+                    # It's only a inactive shim
+                    continue
+
+        # We don't have such a command
+        raise CommandFailedException(
+            'command not in PATH, tried: %s' %
+            ' '.join(cls.COMMAND_ALTERNATIVES))
+
     @classmethod
     def get_version_info(cls, path=None):
         """
@@ -738,7 +803,7 @@ class PostgreSQLClient(Command):
 
         :param str path: the PATH env
         """
-        if cls.COMMAND is None:
+        if cls.COMMAND_ALTERNATIVES is None:
             raise NotImplementedError(
                 "get_version_info cannot be invoked on %s" % cls.__name__)
 
@@ -749,16 +814,21 @@ class PostgreSQLClient(Command):
 
         # Get the version string
         try:
-            command = Command(cls.COMMAND, path=path, check=True)
-            version_info['full_path'] = command.cmd
-            command("--version")
+            command = cls.find_command(path)
         except CommandFailedException as e:
-            _logger.debug("Error invoking %s: %s", cls.COMMAND, e)
+            _logger.debug("Error invoking %s: %s", cls.__name__, e)
             return version_info
 
+        version_info['full_path'] = command.cmd
         # Parse the full text version
-        full_version = command.out.strip().split()[-1]
-        version_info['full_version'] = Version(full_version)
+        try:
+            full_version = command.out.strip().split()[-1]
+            version_info['full_version'] = Version(full_version)
+        except IndexError:
+            _logger.debug("Error parsing %s version output",
+                          version_info['full_path'])
+            return version_info
+
         # Extract the major version
         version_info['major_version'] = Version(barman.utils.simplify_version(
             full_version))
@@ -771,12 +841,12 @@ class PgBaseBackup(PostgreSQLClient):
     Wrapper class for the pg_basebackup system command
     """
 
-    COMMAND = 'pg_basebackup'
+    COMMAND_ALTERNATIVES = ['pg_basebackup']
 
     def __init__(self,
                  connection,
                  destination,
-                 command=COMMAND,
+                 command,
                  version=None,
                  app_name=None,
                  bwlimit=None,
@@ -810,6 +880,13 @@ class PgBaseBackup(PostgreSQLClient):
         # Set the backup destination
         self.args += ['-v', '--no-password', '--pgdata=%s' % destination]
 
+        if version and version >= Version("10"):
+            # If version of the client is >= 10 it would use
+            # a temporary replication slot by default to keep WALs.
+            # We don't need it because Barman already stores the full
+            # WAL stream, so we disable this feature to avoid wasting one slot.
+            self.args += ['--no-slot']
+
         # The tablespace mapping option is repeated once for each tablespace
         if tbs_mapping:
             for (tbs_source, tbs_destination) in tbs_mapping.items():
@@ -834,12 +911,12 @@ class PgReceiveXlog(PostgreSQLClient):
     Wrapper class for pg_receivexlog
     """
 
-    COMMAND = "pg_receivexlog"
+    COMMAND_ALTERNATIVES = ["pg_receivewal", "pg_receivexlog"]
 
     def __init__(self,
                  connection,
                  destination,
-                 command=COMMAND,
+                 command,
                  version=None,
                  app_name=None,
                  synchronous=False,
@@ -936,3 +1013,51 @@ class BarmanSubProcess(object):
             stdin=devnull, stdout=devnull, stderr=devnull)
         _logger.debug("BarmanSubProcess: subprocess started. pid: %s",
                       proc.pid)
+
+
+def shell_quote(arg):
+    """
+    Quote a string argument to be safely included in a shell command line.
+
+    :param str arg: The script argument
+    :return: The argument quoted
+    """
+
+    # This is an excerpt of the Bash manual page, and the same applies for
+    # every Posix compliant shell:
+    #
+    #     A  non-quoted backslash (\) is the escape character.  It preserves
+    #     the literal value of the next character that follows, with the
+    #     exception of <newline>.  If a \<newline> pair appears, and the
+    #     backslash is not itself quoted, the \<newline> is treated as a
+    #     line continuation (that is, it is removed  from  the  input
+    #     stream  and  effectively ignored).
+    #
+    #     Enclosing characters in single quotes preserves the literal value
+    #     of each character within the quotes.  A single quote may not occur
+    #     between single quotes, even when pre-ceded by a backslash.
+    #
+    # This means that, as long as the original string doesn't contain any
+    # apostrophe character, it can be safely included between single quotes.
+    #
+    # If a single quote is contained in the string, we must terminate the
+    # string with a quote, insert an apostrophe character escaping it with
+    # a backslash, and then start another string using a quote character.
+
+    assert arg is not None
+    return "'%s'" % arg.replace("'", "'\\''")
+
+
+def full_command_quote(command, args=None):
+    """
+    Produce a command with quoted arguments
+
+    :param str command: the command to be executed
+    :param list[str] args: the command arguments
+    :rtype: str
+    """
+    if args is not None and len(args) > 0:
+        return "%s %s" % (
+            command, ' '.join([shell_quote(arg) for arg in args]))
+    else:
+        return command
