@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2016 2ndQuadrant Italia Srl
+# Copyright (C) 2011-2017 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -58,6 +58,12 @@ class WALFileException(BarmanException):
     """
     Base exception for all the errors related to WAL files.
     """
+    def __str__(self):
+        """
+        Human readable string representation
+        """
+        return "%s:%s" % (self.__class__.__name__,
+                          self.args[0] if self.args else None)
 
 
 class HookScriptException(BarmanException):
