@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2016 2ndQuadrant Italia Srl
+# Copyright (C) 2013-2017 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -443,6 +443,9 @@ class BackupInfo(FieldListFile):
     included_files = Field('included_files',
                            load=ast.literal_eval, dump=null_repr)
     backup_label = Field('backup_label', load=ast.literal_eval, dump=null_repr)
+    copy_stats = Field('copy_stats', load=ast.literal_eval, dump=null_repr)
+    xlog_segment_size = Field('xlog_segment_size', load=int,
+                              default=xlog.DEFAULT_XLOG_SEG_SIZE)
 
     __slots__ = ('server', 'config', 'backup_manager',
                  'backup_id', 'backup_version')
@@ -503,8 +506,10 @@ class BackupInfo(FieldListFile):
         """
         Get the list of required WAL segments for the current backup
         """
-        return xlog.generate_segment_names(self.begin_wal, self.end_wal,
-                                           self.version)
+        return xlog.generate_segment_names(
+            self.begin_wal, self.end_wal,
+            self.version,
+            self.xlog_segment_size)
 
     def get_list_of_files(self, target):
         """
@@ -639,7 +644,9 @@ class BackupInfo(FieldListFile):
         """
         result = dict(self.items())
         result.update(backup_id=self.backup_id, server_name=self.server_name,
-                      mode=self.mode, tablespaces=self.tablespaces)
+                      mode=self.mode, tablespaces=self.tablespaces,
+                      included_files=self.included_files,
+                      copy_stats=self.copy_stats)
         return result
 
     def to_json(self):
