@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2016 2ndQuadrant Italia Srl
+# Copyright (C) 2011-2017 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -126,11 +126,10 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
         :raise ArchiverFailure: when something goes wrong
         """
 
-    def archive(self, fxlogdb, verbose=True):
+    def archive(self, verbose=True):
         """
         Archive WAL files, discarding duplicates or those that are not valid.
 
-        :param file fxlogdb: File object for xlogdb interactions
         :param boolean verbose: Flag for verbose output
         """
         compressor = self.backup_manager.compression_manager.get_compressor()
@@ -218,12 +217,6 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
                                                 wal_info.name,
                                                 e))
                 return
-            # Updates the information of the WAL archive with
-            # the latest segments
-            fxlogdb.write(wal_info.to_xlogdb_line())
-            # flush and fsync for every line
-            fxlogdb.flush()
-            os.fsync(fxlogdb.fileno())
 
         if processed:
             _logger.debug("Archived %s out of %s xlog segments from %s for %s",
@@ -322,36 +315,48 @@ class WalArchiver(with_metaclass(ABCMeta, RemoteStatusMixin)):
             # Compress the file only if not already compressed
             if compressor and not wal_info.compression:
                 compressor.compress(src_file, tmp_file)
-                shutil.copystat(src_file, tmp_file)
-                os.rename(tmp_file, dst_file)
-                os.unlink(src_file)
-                # Update wal_info
-                stat = os.stat(dst_file)
-                wal_info.size = stat.st_size
-                wal_info.compression = compressor.compression
-            else:
-                # Try to atomically rename the file. If successful,
-                # the renaming will be an atomic operation
-                # (this is a POSIX requirement).
-                try:
-                    os.rename(src_file, dst_file)
-                except OSError:
-                    # Source and destination are probably on different
-                    # filesystems
-                    shutil.copy2(src_file, tmp_file)
+
+            # Perform the real filesystem operation with the xlogdb lock taken.
+            # This makes the operation atomic from the xlogdb file POV
+            with self.server.xlogdb('a') as fxlogdb:
+                if compressor and not wal_info.compression:
+                    shutil.copystat(src_file, tmp_file)
                     os.rename(tmp_file, dst_file)
                     os.unlink(src_file)
-            # At this point the original file has been removed
-            wal_info.orig_filename = None
+                    # Update wal_info
+                    stat = os.stat(dst_file)
+                    wal_info.size = stat.st_size
+                    wal_info.compression = compressor.compression
+                else:
+                    # Try to atomically rename the file. If successful,
+                    # the renaming will be an atomic operation
+                    # (this is a POSIX requirement).
+                    try:
+                        os.rename(src_file, dst_file)
+                    except OSError:
+                        # Source and destination are probably on different
+                        # filesystems
+                        shutil.copy2(src_file, tmp_file)
+                        os.rename(tmp_file, dst_file)
+                        os.unlink(src_file)
+                # At this point the original file has been removed
+                wal_info.orig_filename = None
+
+                # Execute fsync() on the archived WAL file
+                file_fd = os.open(dst_file, os.O_RDONLY)
+                os.fsync(file_fd)
+                os.close(file_fd)
+                # Execute fsync() on the archived WAL containing directory
+                fsync_dir(dst_dir)
+                # Execute fsync() also on the incoming directory
+                fsync_dir(src_dir)
+                # Updates the information of the WAL archive with
+                # the latest segments
+                fxlogdb.write(wal_info.to_xlogdb_line())
+                # flush and fsync for every line
+                fxlogdb.flush()
+                os.fsync(fxlogdb.fileno())
 
-            # Execute fsync() on the archived WAL file
-            file_fd = os.open(dst_file, os.O_RDONLY)
-            os.fsync(file_fd)
-            os.close(file_fd)
-            # Execute fsync() on the archived WAL containing directory
-            fsync_dir(dst_dir)
-            # Execute fsync() also on the incoming directory
-            fsync_dir(src_dir)
         except Exception as e:
             # In case of failure save the exception for the post scripts
             error = e
@@ -465,7 +470,6 @@ class FileWalArchiver(WalArchiver):
         pg_stat_archiver = postgres.get_archiver_stats()
         if pg_stat_archiver is not None:
             result.update(pg_stat_archiver)
-
         return result
 
     def get_next_batch(self):
@@ -487,6 +491,9 @@ class FileWalArchiver(WalArchiver):
         files = []
         errors = []
         for file_name in file_names:
+            # Ignore temporary files
+            if file_name.endswith('.tmp'):
+                continue
             if xlog.is_any_xlog_file(file_name) and os.path.isfile(file_name):
                 files.append(file_name)
             else:
@@ -506,35 +513,38 @@ class FileWalArchiver(WalArchiver):
         :param CheckStrategy check_strategy: the strategy for the management
              of the results of the various checks
         """
+        check_strategy.init_check('archive_mode')
         remote_status = self.get_remote_status()
         # If archive_mode is None, there are issues connecting to PostgreSQL
         if remote_status['archive_mode'] is None:
             return
         # Check archive_mode parameter: must be on
         if remote_status['archive_mode'] in ('on', 'always'):
-            check_strategy.result(self.config.name, 'archive_mode', True)
+            check_strategy.result(self.config.name, True)
         else:
             msg = "please set it to 'on'"
             if self.server.postgres.server_version >= 90500:
                 msg += " or 'always'"
-            check_strategy.result(self.config.name, 'archive_mode', False, msg)
-
+            check_strategy.result(self.config.name, False, hint=msg)
+        check_strategy.init_check('archive_command')
         if remote_status['archive_command'] and \
                 remote_status['archive_command'] != '(disabled)':
-            check_strategy.result(self.config.name, 'archive_command',
-                                  True)
+            check_strategy.result(self.config.name,
+                                  True,
+                                  check='archive_command')
 
             # Report if the archiving process works without issues.
             # Skip if the archive_command check fails
             # It can be None if PostgreSQL is older than 9.4
             if remote_status.get('is_archiving') is not None:
                 check_strategy.result(
-                    self.config.name, 'continuous archiving',
-                    remote_status['is_archiving'])
+                    self.config.name,
+                    remote_status['is_archiving'],
+                    check='continuous archiving')
         else:
             check_strategy.result(
-                self.config.name, 'archive_command', False,
-                'please set it accordingly to documentation')
+                self.config.name, False,
+                hint='please set it accordingly to documentation')
 
     def status(self):
         """
@@ -680,7 +690,8 @@ class StreamingWalArchiver(WalArchiver):
         streaming_status = self.server.streaming.get_remote_status()
         if streaming_status["streaming_supported"] is None:
             raise ArchiverFailure(
-                'failed opening the PostgreSQL streaming connection')
+                'failed opening the PostgreSQL streaming connection '
+                'for server %s' % (self.config.name))
         elif not streaming_status["streaming_supported"]:
             raise ArchiverFailure(
                 'PostgreSQL version too old (%s < 9.2)' %
@@ -793,6 +804,9 @@ class StreamingWalArchiver(WalArchiver):
         skip = []
         errors = []
         for file_name in file_names:
+            # Ignore temporary files
+            if file_name.endswith('.tmp'):
+                continue
             if not os.path.isfile(file_name):
                 errors.append(file_name)
             elif xlog.is_partial_file(file_name):
@@ -802,12 +816,13 @@ class StreamingWalArchiver(WalArchiver):
             else:
                 errors.append(file_name)
         # In case of more than a partial file, keep the last
-        # and treat the rest as errors
+        # and treat the rest as normal files
         if len(skip) > 1:
-            errors.extend(skip[:-1])
-            _logger.warning('Multiple partial files found for server %s: %s' %
-                            (self.config.name,
-                             ", ".join([os.path.basename(f) for f in errors])))
+            partials = skip[:-1]
+            _logger.info('Archiving partial files for server %s: %s' %
+                         (self.config.name,
+                          ", ".join([os.path.basename(f) for f in partials])))
+            files.extend(partials)
             skip = skip[-1:]
 
         # Keep the last full WAL file in case no partial file is present
@@ -822,19 +837,21 @@ class StreamingWalArchiver(WalArchiver):
 
     def check(self, check_strategy):
         """
-        Perform additional checks for FileWalArchiver - invoked
+        Perform additional checks for StreamingWalArchiver - invoked
         by server.check_postgres
 
         :param CheckStrategy check_strategy: the strategy for the management
              of the results of the various checks
         """
 
+        check_strategy.init_check('pg_receivexlog')
         # Check the version of pg_receivexlog
         remote_status = self.get_remote_status()
         check_strategy.result(
-            self.config.name, 'pg_receivexlog',
+            self.config.name,
             remote_status['pg_receivexlog_installed'])
         hint = None
+        check_strategy.init_check('pg_receivexlog compatible')
         if not remote_status['pg_receivexlog_compatible']:
             pg_version = 'Unknown'
             if self.server.streaming is not None:
@@ -843,9 +860,9 @@ class StreamingWalArchiver(WalArchiver):
                 pg_version,
                 remote_status['pg_receivexlog_version']
             )
-        check_strategy.result(
-            self.config.name, 'pg_receivexlog compatible',
-            remote_status['pg_receivexlog_compatible'], hint=hint)
+        check_strategy.result(self.config.name,
+                              remote_status['pg_receivexlog_compatible'],
+                              hint=hint)
 
         # Check if pg_receivexlog is running, by retrieving a list
         # of running 'receive-wal' processes from the process manager.
@@ -853,13 +870,15 @@ class StreamingWalArchiver(WalArchiver):
 
         # If there's at least one 'receive-wal' process running for this
         # server, the test is passed
+        check_strategy.init_check('receive-wal running')
         if receiver_list:
             check_strategy.result(
-                self.config.name, 'receive-wal running', True)
+                self.config.name, True)
         else:
             check_strategy.result(
-                self.config.name, 'receive-wal running', False,
-                'See the Barman log file for more details')
+                self.config.name,
+                False,
+                hint='See the Barman log file for more details')
 
     def _is_synchronous(self):
         """
